@@ -2,6 +2,7 @@ package org.debian.maven.packager;
 
 /*
  * Copyright 2009 Ludovic Claude.
+ * Copyright 2011 Damien Raude-Morvan.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -943,6 +944,33 @@ public class DependenciesSolver {
         }
 
         POMInfo pom = getRepository().searchMatchingPOM(dependency);
+        try {
+            if (pom == null && dependency.getVersion() == null && getPOM(sourcePom).getParent() != null) {
+                String version = getPOM(sourcePom).getVersionFromManagementDependency(dependency);
+                dependency.setVersion(version);
+                pom = getRepository().searchMatchingPOM(dependency);
+            }
+        } catch (Exception e) {
+            e.printStackTrace();
+        }
+        if (pom == null && dependency.getVersion() != null) {
+            List poms = getRepository().searchMatchingPOMsIgnoreVersion(dependency);
+            for (Iterator i = poms.iterator(); i.hasNext(); ) {
+                POMInfo potentialPom = (POMInfo) i.next();
+                String mavenRules = (String) potentialPom.getProperties().get("debian.mavenRules");
+                if (mavenRules != null) {
+                    StringTokenizer st = new StringTokenizer(mavenRules, ",");
+                    while (st.hasMoreTokens()) {
+                        String ruleDef = st.nextToken().trim();
+                        DependencyRule rule = new DependencyRule(ruleDef);
+                        if (rule.matches(dependency) && rule.apply(dependency).equals(potentialPom.getThisPom())) {
+                            pom = potentialPom;
+                            pomTransformer.getRules().add(rule);
+                        }
+                    }
+                }
+            }
+        }
         if (pom == null && dependency.getVersion() == null) {
             // Set a dummy version and try again
             for (int version = 0; version < 10; version++) {
@@ -1040,6 +1068,8 @@ public class DependenciesSolver {
                         System.out.print("[y]/n > ");
                         String s = readLine().trim().toLowerCase();
                         if (!s.startsWith("n")) {
+                            System.out.println("Rescanning /usr/share/maven-repo...");
+                            pomTransformer.getRepository().scan();
                             return resolveDependency(dependency, sourcePom, buildTime, mavenExtension, management);
                         } 
                     }
@@ -1080,27 +1110,29 @@ public class DependenciesSolver {
             if (pom.getOriginalVersion() != null && (pom.getProperties().containsKey("debian.hasPackageVersion"))) {
                 libraryWithVersionConstraint += " (>= " + version + ")";
             }
-            if (buildTime) {
-                if ("test".equals(dependency.getScope())) {
-                    testDepends.add(libraryWithVersionConstraint);
-                } else if ("maven-plugin".equals(dependency.getType())) {
-                    if (!packageType.equals("ant")) {
-                        compileDepends.add(libraryWithVersionConstraint);
-                    }
-                } else if (mavenExtension) {
-                    if (!packageType.equals("ant")) {
+            if (!management) {
+                if (buildTime) {
+                    if ("test".equals(dependency.getScope())) {
+                        testDepends.add(libraryWithVersionConstraint);
+                    } else if ("maven-plugin".equals(dependency.getType())) {
+                        if (!packageType.equals("ant")) {
+                            compileDepends.add(libraryWithVersionConstraint);
+                        }
+                    } else if (mavenExtension) {
+                        if (!packageType.equals("ant")) {
+                            compileDepends.add(libraryWithVersionConstraint);
+                        }
+                    } else {
                         compileDepends.add(libraryWithVersionConstraint);
                     }
                 } else {
-                    compileDepends.add(libraryWithVersionConstraint);
-                }
-            } else {
-                if (dependency.isOptional()) {
-                    optionalDepends.add(libraryWithVersionConstraint);
-                } else if ("test".equals(dependency.getScope())) {
-                    testDepends.add(libraryWithVersionConstraint);
-                } else {
-                    runtimeDepends.add(libraryWithVersionConstraint);
+                    if (dependency.isOptional()) {
+                        optionalDepends.add(libraryWithVersionConstraint);
+                    } else if ("test".equals(dependency.getScope())) {
+                        testDepends.add(libraryWithVersionConstraint);
+                    } else {
+                        runtimeDepends.add(libraryWithVersionConstraint);
+                    }
                 }
             }
             versionedPackagesAndDependencies.put(libraryWithVersionConstraint, dependency);
@@ -1287,19 +1319,23 @@ public class DependenciesSolver {
             System.out.println("  -m<repo root>--maven-repo=<repo root>: location of the Maven repository,");
             System.out.println("    used to force the versions of the Maven plugins used in the current");
             System.out.println("    POM file with the versions found in the repository");
+            System.out.println("  --base-directory: path to root directory of package");
+            System.out.println("  --non-explore: doesn't explore directories for pom.xml");
             return;
         }
-        DependenciesSolver solver = new DependenciesSolver();
-
-        solver.setBaseDir(new File("."));
-        solver.setExploreProjects(true);
-        solver.setOutputDirectory(new File("debian"));
-
-        int i = inc(-1, args);
+        
+        // Default values
         boolean verbose = false;
         String debianPackage = "";
         String packageType = "maven";
         File mavenRepo = null;
+        File baseDirectory = new File(".");
+        
+        DependenciesSolver solver = new DependenciesSolver();
+        solver.setExploreProjects(true); // can be overriden by args
+        
+        // Parse parameters
+        int i = inc(-1, args);
         while (i < args.length && (args[i].trim().startsWith("-") || args[i].trim().isEmpty())) {
             String arg = args[i].trim();
             if ("--verbose".equals(arg) || "-v".equals(arg)) {
@@ -1322,14 +1358,21 @@ public class DependenciesSolver {
                 mavenRepo = new File(arg.substring(2));
             } else if (arg.startsWith("--maven-repo=")) {
                 mavenRepo = new File(arg.substring("--maven-repo=".length()));
+            } else if (arg.startsWith("-b")) {
+                baseDirectory = new File(arg.substring(2));
+            } else if (arg.startsWith("--base-directory=")) {
+            	baseDirectory = new File(arg.substring("--base-directory=".length()));
+            } else if (arg.equals("--non-explore")) {
+            	solver.setExploreProjects(false);
             }
             i = inc(i, args);
         }
-        File poms = new File(solver.getOutputDirectory(), debianPackage + ".poms");
 
+        solver.setBaseDir(baseDirectory);
+        solver.setOutputDirectory(new File(baseDirectory, "debian"));
         solver.setPackageName(debianPackage);
         solver.setPackageType(packageType);
-        solver.setExploreProjects(true);
+        File poms = new File(solver.getOutputDirectory(), debianPackage + ".poms");
         solver.setListOfPoms(poms);
 
         if (mavenRepo != null) {
